/*
Pulse and Sample

wombatpi.net

Modified 14-Apr-2024


Notes:  
The Sample window is sensitive to all digital and analog IO the occurs during,
Thus is enclosed in a no-interrupts block. 
The audio output A0 feeds to high impedance (i.e > 4k) audio amp

*/

#include <arduino.h>
//#include "pwm.h"
#include "FspTimer.h"
#include "wombat_analog.h"
#include "target_sense.h"



FspTimer pulse_timer;


// number of samples to take in sample window
//
volatile int sampleCounter = 0;
volatile uint16_t sampleArray[SAMPLE_COUNT_MAX];

// Do pulse and sampling in one tight function
// 
void timerADCReadInterrupt(timer_callback_args_t __attribute((unused)) *p_args)
{
  int sampleCounter;      
  noInterrupts();  

  *PFS_P104PFS_BY = 0x05;      // Pulse on D3 
  //
  //  
  for(sampleCounter = 0; sampleCounter < PULSE_WIDTH; sampleCounter++)
  {
    asm("NOP");
  }  
  *PFS_P104PFS_BY = 0x04;      // Pulse off D3  

  // Sampling
  //    
  for(sampleCounter = 0 ; sampleCounter < SAMPLE_COUNT_MAX; sampleCounter++)
  {      
    //*PFS_P107PFS_BY = 0x05;         // digitalWrite(monitorPinD7, HIGH)  

    *ADC140_ADCSR |= (0x01 << 15);  // Next ADC conversion = write to register c. 300nS
    while((*ADC140_ADCSR &= (0x01 << 15) ) != 0x0000);  // if things not setup right, endless loop
    
    sampleArray[sampleCounter] = *ADC140_ADDR01;      
    //*PFS_P107PFS_BY = 0x04;         // digitalWrite(monitorPin D7, LOW);     
  }  
     
  interrupts();    
  sampleReady = true;   
}



// coil pulse and sample using timer, pin D3
//

bool setupPulse()
{
  uint8_t timer_type = GPT_TIMER;
  int8_t tindex = FspTimer::get_available_timer(timer_type);
  
  pinMode(D3, OUTPUT);

  if (tindex < 0){
    tindex = FspTimer::get_available_timer(timer_type, true);
  }
  if (tindex < 0){
    return false;
  }

  FspTimer::force_use_of_pwm_reserved_timer();

  // ie 600 Hz
  //  
  if(!pulse_timer.begin(TIMER_MODE_PERIODIC, timer_type, tindex, PULSE_FREQUENCY, 0.0f, timerADCReadInterrupt)){
    return false;
  }  

  if (!pulse_timer.setup_overflow_irq()){
    return false;
  }

  if (!pulse_timer.open()){
    return false;
  }

  if (!pulse_timer.start()){
    return false;
  }
  return true;
}



void setupSample() {

  pinMode(analogPin, INPUT);  
  analogReference(AR_EXTERNAL);
  
  analogReadResolution(14);        // This code reads ADC result directly, so this setup not needed
  *DAC12_DAADSCR |= (0x01 << 7);     // set the D/A A/D sync bit 
  
  uint16_t value = analogRead(analogPin);  // Do a read to get everthing set-up
  // Update registers - do NOT use analogRead() after this

  *ADC140_ADCER = 0x06;              // 14 bit mode (already set), clear ACE bit 5
  
  //-------------------------------------------------------------
 
  *ADC140_ADCSR |= (0x01 << 15);     // Start a conversion
  
  adc_val_16 = 8;  
}

