/*
wombat_dac_audio

wombatpi.net

Modified:  14-April-2024

 From example by ---- TriodeGirl ----

 This code is "AS IS" without warranty or liability. 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.


*/


/*

We use Triangle waves; They sound good and are simple to make

*/

#include <FspTimer.h>

#include "target_sense.h"

FspTimer soundWave;

// middle C
#define freq_c1 (262)
#define freq_e1 (330)
#define freq_g1 (392)
#define freq_c2 (523)


double volume;
double midTone = 250;  // Hz


class TRIANGLEWAVE {

  private:
    float speed;       // added or subtracted every interrupt, calculated using frequency
    float amplitude;   // value between 0 and 1
    float volume = 0.5;    // value between 0 and 1
    int goingup = 1;

    const double HALF_CLOCK_FREQUENCY = 10000.0;  // the interrupt frequency / 2
    uint16_t MIDDLE_VOLTAGE = 1000;  // an offset so signal is an AC signal, fed into amp.
    const float DAC_MULTIPLIER = 1000;   // this is max value of the DAC at volume =1.0,, i.e the peak to peak ~ 1  Volts
      
  public: 
        
    uint16_t  INT_amplitude;
 
    void setVolume(double v)
    {      
      if(v > 1.0)
      {
        volume = 1.0;
      }
      else if( v < 0)
      {
        volume = 0;
      }
      else
      {
        volume = v;
      }
      MIDDLE_VOLTAGE = 1000 - ((DAC_MULTIPLIER/2) * volume); // the middle of the waveform remains the same
    }

    void setFrequency(double f)
    {
      if(f > 3000)
      {
        f = 3000;  // maximum 3 kHz
      }
      speed = (f / HALF_CLOCK_FREQUENCY);  // the added to the amplitude every interrupt      
    }

    // called every interrupt very quickly, i.e at 20kHz !
    //
    void advance()
    {        
      amplitude += (goingup * speed);
      if(amplitude > 1)
      {
        goingup = -1;   // going down, i.e down slope of triangle
        amplitude = 1;  // not ideal
      }
      else if (amplitude < 0)
      {
        amplitude = 0; // not ideal
        goingup = 1;  // going up, i.e up slope of the tirangle
      }
      INT_amplitude = MIDDLE_VOLTAGE;
      INT_amplitude += (uint16_t)(amplitude * volume * DAC_MULTIPLIER);      
    }
};


TRIANGLEWAVE wave1;


// RA4M1 User’s Manual: Hardware
// This doc has all the register discriptions I use:
// https://www.renesas.com/us/en/document/mah/renesas-ra4m1-group-users-manual-hardware

// ARM-developer - Accessing memory-mapped peripherals
// https://developer.arm.com/documentation/102618/0100

// Low Power Mode Control - See datasheet section 10
#define SYSTEM 0x40010000 // System Registers
#define SYSTEM_SBYCR   ((volatile unsigned short *)(SYSTEM + 0xE00C))      // Standby Control Register
#define SYSTEM_MSTPCRA ((volatile unsigned int   *)(SYSTEM + 0xE01C))      // Module Stop Control Register A

#define MSTP 0x40040000 // Module Registers
#define MSTP_MSTPCRB   ((volatile unsigned int   *)(MSTP   + 0x7000))      // Module Stop Control Register B
#define MSTPB2   2  // CAN0
#define MSTPB8   8  // IIC1
#define MSTPB9   9  // IIC0
#define MSTPB18 18  // SPI1
#define MSTPB19 19  // SPI0
#define MSTPB22 22  // SCI9
#define MSTPB29 29  // SCI2
#define MSTPB30 30  // SCI1
#define MSTPB31 31  // SCI0

#define MSTP_MSTPCRC   ((volatile unsigned int   *)(MSTP + 0x7004))        // Module Stop Control Register C
#define MSTP_MSTPCRD   ((volatile unsigned int   *)(MSTP + 0x7008))        // Module Stop Control Register D
#define MSTPD2   2  // AGT1   - Asynchronous General Purpose Timer 1 Module
#define MSTPD3   3  // AGT0   - Asynchronous General Purpose Timer 0 Module
#define MSTPD5   5  // GPT320 and GPT321 General 32 bit PWM Timer Module
#define MSTPD6   6  // GPT162 to GPT167 General 16 bit PWM Timer Module
#define MSTPD14 14  // POEG   - Port Output Enable for GPT Module Stop
#define MSTPD16 16  // ADC140 - 14-Bit A/D Converter Module
#define MSTPD19 19  // DAC8   -  8-Bit D/A Converter Module
#define MSTPD20 20  // DAC12  - 12-Bit D/A Converter Module
#define MSTPD29 29  // ACMPLP - Low-Power Analog Comparator Module
#define MSTPD31 31  // OPAMP  - Operational Amplifier Module

// The Mode Control bits are read as 1, the write value should be 1.
// Bit value 0: Cancel the module-stop state 
// Bit value 1: Enter the module-stop state.


// ====  Asynchronous General Purpose Timer (AGT) =====
#define AGTBASE 0x40084000 
#define AGT0_AGTCR    ((volatile unsigned char  *)(AGTBASE + 0x008))  // AGT Control Register

// 12-Bit D/A Converter
#define DACBASE 0x40050000          // DAC Base - DAC output on A0 (P014 AN09 DAC)
#define DAC12_DADR0    ((volatile unsigned short *)(DACBASE + 0xE000))      // D/A Data Register 0 
#define DAC12_DACR     ((volatile unsigned char  *)(DACBASE + 0xE004))      // D/A Control Register
#define DAC12_DADPR    ((volatile unsigned char  *)(DACBASE + 0xE005))      // DADR0 Format Select Register
#define DAC12_DAADSCR  ((volatile unsigned char  *)(DACBASE + 0xE006))      // D/A A/D Synchronous Start Control Register
#define DAC12_DAVREFCR ((volatile unsigned char  *)(DACBASE + 0xE007))      // D/A VREF Control Register

// =========== Ports ============
// 19.2.5 Port mn Pin Function Select Register (PmnPFS/PmnPFS_HA/PmnPFS_BY) (m = 0 to 9; n = 00 to 15)
#define PORTBASE 0x40040000 /* Port Base */

#define P000PFS 0x0800  // Port 0 Pin Function Select Register
#define PFS_P000PFS ((volatile unsigned int *)(PORTBASE + P000PFS))            // 
#define PFS_P001PFS ((volatile unsigned int *)(PORTBASE + P000PFS + ( 1 * 4))) // 
#define PFS_P002PFS ((volatile unsigned int *)(PORTBASE + P000PFS + ( 2 * 4))) // 
#define PFS_P003PFS ((volatile unsigned int *)(PORTBASE + P000PFS + ( 3 * 4))) // 
#define PFS_P004PFS ((volatile unsigned int *)(PORTBASE + P000PFS + ( 4 * 4))) // 
#define PFS_P005PFS ((volatile unsigned int *)(PORTBASE + P000PFS + ( 5 * 4))) // 
#define PFS_P006PFS ((volatile unsigned int *)(PORTBASE + P000PFS + ( 6 * 4))) // 
#define PFS_P007PFS ((volatile unsigned int *)(PORTBASE + P000PFS + ( 7 * 4))) // 
#define PFS_P008PFS ((volatile unsigned int *)(PORTBASE + P000PFS + ( 8 * 4))) // 
// #define PFS_P009PFS ((volatile unsigned int *)(PORTBASE + P000PFS + ( 9 * 4))) // Does not exist
#define PFS_P010PFS ((volatile unsigned int *)(PORTBASE + P000PFS + (10 * 4))) // 
#define PFS_P011PFS ((volatile unsigned int *)(PORTBASE + P000PFS + (11 * 4))) // 
#define PFS_P012PFS ((volatile unsigned int *)(PORTBASE + P000PFS + (12 * 4))) // 
#define PFS_P013PFS ((volatile unsigned int *)(PORTBASE + P000PFS + (13 * 4))) // 
#define PFS_P014PFS ((volatile unsigned int *)(PORTBASE + P000PFS + (14 * 4))) // A0 / DAC12
#define PFS_P015PFS ((volatile unsigned int *)(PORTBASE + P000PFS + (15 * 4))) // 

#define PFS_P100PFS_BY ((volatile unsigned char  *)(PORTBASE + 0x0843))   // 8 bits - A5
#define PFS_P101PFS_BY ((volatile unsigned char  *)(PORTBASE + 0x0843 + ( 1 * 4))) // A4
#define PFS_P102PFS_BY ((volatile unsigned char  *)(PORTBASE + 0x0843 + ( 2 * 4))) // D5
#define PFS_P103PFS_BY ((volatile unsigned char  *)(PORTBASE + 0x0843 + ( 3 * 4))) // D4
#define PFS_P104PFS_BY ((volatile unsigned char  *)(PORTBASE + 0x0843 + ( 4 * 4))) // D3
#define PFS_P105PFS_BY ((volatile unsigned char  *)(PORTBASE + 0x0843 + ( 5 * 4))) // D2
#define PFS_P106PFS_BY ((volatile unsigned char  *)(PORTBASE + 0x0843 + ( 6 * 4))) // D6
#define PFS_P107PFS_BY ((volatile unsigned char  *)(PORTBASE + 0x0843 + ( 7 * 4))) // D7
#define PFS_P108PFS_BY ((volatile unsigned char  *)(PORTBASE + 0x0843 + ( 8 * 4))) // SWDIO
#define PFS_P109PFS_BY ((volatile unsigned char  *)(PORTBASE + 0x0843 + ( 9 * 4))) // D11 / MOSI
#define PFS_P110PFS_BY ((volatile unsigned char  *)(PORTBASE + 0x0843 + (10 * 4))) // D12 / MISO
#define PFS_P111PFS_BY ((volatile unsigned char  *)(PORTBASE + 0x0843 + (11 * 4))) // D13 / SCLK
#define PFS_P112PFS_BY ((volatile unsigned char  *)(PORTBASE + 0x0843 + (12 * 4))) // D10 / CS
#define PFS_P300PFS_BY ((volatile unsigned char  *)(PORTBASE + 0x08C3))            // SWCLK (P300)
#define PFS_P301PFS_BY ((volatile unsigned char  *)(PORTBASE + 0x08C3 + (01 * 4))) // D0 / RxD (P301)
#define PFS_P302PFS_BY ((volatile unsigned char  *)(PORTBASE + 0x08C3 + (02 * 4))) // D1 / TxD (P302) 
#define PFS_P303PFS_BY ((volatile unsigned char  *)(PORTBASE + 0x08C3 + (03 * 4))) // D9
#define PFS_P304PFS_BY ((volatile unsigned char  *)(PORTBASE + 0x08C3 + (04 * 4))) // D8




void timer_DACOut_Interrupt(timer_callback_args_t __attribute((unused)) *p_args)
{
  wave1.advance();   
  *DAC12_DADR0 = wave1.INT_amplitude;         // DAC update            - takes c. 210nS  - DAC ignores top 4 bits
}


void setup_dac(void)       // Note make sure ADC is stopped before setup DAC
{
  *MSTP_MSTPCRD &= ~(0x01 << MSTPD20);  // Enable DAC12 module
  *DAC12_DADPR    = 0x00;               // DADR0 Format Select Register - Set right-justified format
  *DAC12_DAADSCR  = 0x80;               // D/A A/D Synchronous Start Control Register - Enable
//  *DAC12_DAADSCR  = 0x00;             // D/A A/D Synchronous Start Control Register - Default
// 36.3.2 Notes on Using the Internal Reference Voltage as the Reference Voltage
  *DAC12_DAVREFCR = 0x00;               // D/A VREF Control Register - Write 0x00 first - see 36.2.5
  *DAC12_DADR0    = 0x0000;             // D/A Data Register 0 
   delayMicroseconds(10);               // Needed delay - see datasheet
  *DAC12_DAVREFCR = 0x01;               // D/A VREF Control Register - Select AVCC0/AVSS0 for Vref
  *DAC12_DACR     = 0x5F;               // D/A Control Register - 
   delayMicroseconds(5);                // 
  *DAC12_DADR0    = 0x0800;             // D/A Data Register 0 
  *PFS_P014PFS   = 0x00000000;          // Port Mode Control - Make sure all bits cleared
  *PFS_P014PFS  |= (0x1 << 15);         // ... use as an analog pin
  }


// called from main every coil pulse, i.e at 500 Hz
// compensate for missed audio interrupt by advancing the waveform ~ 100uSec
//
void compensate_audio_for_pulse()
{
  wave1.advance();      
}




// Startup tune
//
void startTune() 
{
   wave1.setVolume(0.5);
  
   wave1.setFrequency(freq_c1);  
   delay(300);
   wave1.setFrequency(freq_e1);     
   delay(300);
   wave1.setFrequency(freq_c2);
   delay(300); 
   wave1.setFrequency(freq_c1);
}


// Main sound setup
// Setup the DAC, outputing on A0
// Setup the 20kHz timer that outputs the DAC waveform
// 
//
bool setup_soundWave()
{
  uint8_t timer_type = GPT_TIMER;
  int8_t tindex = FspTimer::get_available_timer(timer_type);

  wave1.setVolume(1.0);
  wave1.setFrequency(300);  //Hz

  setup_dac();

  if (tindex < 0){
    tindex = FspTimer::get_available_timer(timer_type, true);
  }
  if (tindex < 0){
    return false;
  }

  //FspTimer::force_use_of_pwm_reserved_timer();

  // 20kHz
  //  
  if(!soundWave.begin(TIMER_MODE_PERIODIC, timer_type, tindex, 20000, 0.0f, timer_DACOut_Interrupt)){
    return false;
  }  

  if (!soundWave.setup_overflow_irq()){
    return false;
  }

  if (!soundWave.open()){
    return false;
  }

  if (!soundWave.start()){
    return false;
  }

  startTune() ;
  return true;
}




// adjust the sound according to signal
// ! Called at 100Hz !
//
void soundAlgorithm2(double signal)
{  
  static int startupCounter = 0;  // allow averages to wind up first,so don't blast ears with loud signal

  double tempF = signal;
  static int tone_test_count = 0;
  static float newVolume = 0;
  
  // change the tone
  //
  tempF = midTone + (midTone * signal  * 0.02);
  
  if(tempF < midTone)
  {
    tempF = midTone;
  }

  // adjust for Target ID
  //
  if(TARGET_SENSE::targetID == OK_BIG)
  {
    tempF += 60; // increase tone
  } 
  else if (TARGET_SENSE::targetID == Fe)
  {
    tempF -= 30;  // decrease the tone
  }
  
  
  if(tempF > (midTone * 2.0))
  {
    tempF = midTone * 2.0;
  }  
  
  wave1.setFrequency(tempF);  //Hz;   

  // volume
  //
  tempF = (signal * 2.0 );  

  // slight increase in volume for boolean IS_TARGET
  //
  if(TARGET_SENSE::targetID == OK_BIG)
  {
    tempF += 5;
  }  

  if(tempF < 0)
  {
    tempF *= -1.0;
  }
  // value is always positive 
  
  if(tempF > 99.0)
  {
    tempF = 99.0;
  }

  tempF = tempF-newVolume;


  // limit volume step
  //
  if (tempF > 4)
  {
    tempF = 4;
  }
  else if (tempF < -4 )
  {
    tempF = -4;
  }

  newVolume += tempF;

  if(startupCounter < 1500)
  {
    startupCounter++;
    newVolume = 20;
  }
  volume = newVolume;
  volume /= 100.0;
  volume += 0.008;

  wave1.setVolume(volume);    
}



